;;; -*- lexical-binding: t -*-

(require 'ert)

(defun proper-list-length (object)
  "Return OBJECT's length if it is a proper list, nil otherwise.
A proper list is neither circular nor dotted (i.e., its last cdr
is nil)."
  (and (listp object) (ignore-errors (length object))))

(let* ((reps     10000)
       (length   256)
       (proper   (make-list length 0))
       (dotted   (append proper 0))
       (circular (nconc (copy-sequence proper) (nthcdr (/ length 2) proper)))
       (lists    (list proper dotted circular)))
  (dolist (fn '(proper-list-length
                format-proper-list-p
                ert--proper-list-p))
    (apply #'message "`%s'\n  proper   %s\n  dotted   %s\n  circular %s"
           fn (mapcar (lambda (list)
                        (garbage-collect)
                        (benchmark-run reps
                          (funcall fn list)))
                      lists))))
